@@ -1841,7 +1841,15 @@ impl CopierWorker {
             / PATROL_TOUCH_PERIOD.as_secs_f64())
         .clamp(0.0, 1.0);
         let tap_file = replication_buffer::construct_tapped_manifest_path(spool_path, source)?;
-        let (mut chunks, base) = parse_manifest_chunks(&tap_file, &targets.replication_targets)?;
+
+        // Get the chunks, but replace everything that's bundled with
+        // the manifest with a zero fingerprint: we don't have to
+        // fetch bundled chunks, just like zero chunks.
+        let (mut chunks, base) = parse_manifest_chunks(
+            &tap_file,
+            &targets.replication_targets,
+            crate::loader::zero_fingerprint(),
+        )?;
         let mut rng = rand::thread_rng();
 
         if let Some(base) = base.as_ref() {
@@ -1852,6 +1860,10 @@ impl CopierWorker {
         // found (or `None`).
         *recent_base_chunk.lock().unwrap() = base;
 
+        // Don't bother keeping alive data for fingerprints we don't
+        // need to fetch.
+        chunks.retain(|fp| !crate::loader::is_well_known_fingerprint(*fp));
+
         // Touch that fraction of the chunks list, with randomised
         // rounding for any fractional number of chunks: rounding
         // down might consistently round to 0, and rounding up
